unit ISAPIFilter;
{DEFINE DEBUG}
interface

uses
  Windows, Messages, SysUtils, Classes, SyncObjs, DateUtils, ISAPI2;

////////////////////////////////////////////////////////////////////////////////
//              Event Notification Order - see MSDN for details...
////////////////////////////////////////////////////////////////////////////////
//
//    01. SF_NOTIFY_READ_RAW_DATA
//    02. SF_NOTIFY_PREPROC_HEADERS
//    03. SF_NOTIFY_URL_MAP
//    04. SF_NOTIFY_AUTHENTICATION
//    05. SF_NOTIFY_AUTH_COMPLETE
//    06. SF_NOTIFY_READ_RAW_DATA
//    07. SF_NOTIFY_SEND_RESPONSE
//    08. SF_NOTIFY_SEND_RAW_DATA
//    09. SF_NOTIFY_END_OF_REQUEST
//    10. SF_NOTIFY_LOG
//    11. SF_NOTIFY_END_OF_NET_SESSION
//
////////////////////////////////////////////////////////////////////////////////

const
  //////////////////////////////////////////////////////////////////////////////
  SF_NOTIFY_SECURE_PORT               = $00000001;
  SF_NOTIFY_NONSECURE_PORT            = $00000002;
  SF_NOTIFY_READ_RAW_DATA             = $00008000;
  SF_NOTIFY_PREPROC_HEADERS           = $00004000;
  SF_NOTIFY_AUTHENTICATION            = $00002000;
  SF_NOTIFY_URL_MAP                   = $00001000;
  SF_NOTIFY_ACCESS_DENIED             = $00000800;
  SF_NOTIFY_SEND_RESPONSE             = $00000040;
  SF_NOTIFY_SEND_RAW_DATA             = $00000400;
  SF_NOTIFY_LOG                       = $00000200;
  SF_NOTIFY_END_OF_REQUEST            = $00000080;
  SF_NOTIFY_END_OF_NET_SESSION        = $00000100;
  SF_NOTIFY_AUTH_COMPLETE             = $04000000;
  SF_NOTIFY_EXTENSION_TRIGGER         = $02000000;
  //////////////////////////////////////////////////////////////////////////////

type
  //////////////////// NEW STRUCTURES IN IIS 5.1 ///////////////////////////////

  //////////////////////////////////////////////  HTTP_FILTER_AUTH_COMPLETE_INFO
  TGetUserTokenProc  = function (var pfc       : THTTP_FILTER_CONTEXT;
                                     phToken   : PHandle): BOOL stdcall;
  PHTTP_FILTER_AUTH_COMPLETE_INFO = ^THTTP_FILTER_AUTH_COMPLETE_INFO;
  THTTP_FILTER_AUTH_COMPLETE_INFO = record
    GetHeader    : TGetHeaderProc;
    SetHeader    : TSetHeaderProc;
    AddHeader    : TAddHeaderProc;
    GetUserToken : TGetUserTokenProc;
    HttpStatus   : DWORD;
    fResetAuth   : BOOL;
    dwReserved   : DWORD;
  end;
  //////////////////////////////////////////////////////////////////////////////

  //////////////////////////////////////////////       HTTP_FILTER_SEND_RESPONSE
  HTTP_FILTER_SEND_RESPONSE   = THTTP_FILTER_PREPROC_HEADERS;
  PHTTP_FILTER_SEND_RESPONSE  = ^HTTP_FILTER_SEND_RESPONSE;
  {$EXTERNALSYM HTTP_FILTER_SEND_RESPONSE}
  THTTP_FILTER_SEND_RESPONSE  = HTTP_FILTER_SEND_RESPONSE;
  LPHTTP_FILTER_SEND_RESPONSE = PHTTP_FILTER_SEND_RESPONSE;
  {$EXTERNALSYM HTTP_FILTER_SEND_RESPONSE}
  //////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////// HTTP_FILTER_EXTENSION_TRIGGER_INFO
  PHTTP_FILTER_EXTENSION_TRIGGER_INFO = ^THTTP_FILTER_EXTENSION_TRIGGER_INFO;
  THTTP_FILTER_EXTENSION_TRIGGER_INFO = record
    dwTriggerType     : DWORD;
    pvTriggerContext  : Pointer;
  end;
  //////////////////////////////////////////////////////////////////////////////

  TOnGetFilterVersion = Procedure(var pVer: THTTP_FILTER_VERSION; Var Result :Bool) Of Object;
  TOnIISNotification  = Procedure(var pFc: THTTP_FILTER_CONTEXT; NotificationType: DWORD; pvNotification: Pointer; Var Result: DWord) Of Object;
  TOnTerminateFilter  = Procedure(dwFlags: DWORD; var Result : Bool) Of Object;

  TOnReadRawData      = TOnIISNotification;
  TOnPreprocHeaders   = TOnIISNotification;
  TOnURLMap           = TOnIISNotification;
  TOnAuthentication   = TOnIISNotification;
  TOnAuthComplete     = TOnIISNotification;
  TOnAccessDenied     = TOnIISNotification;
  TOnSendResponce     = TOnIISNotification;
  TOnSendRawData      = TOnIISNotification;
  TOnEndOfRequest     = TOnIISNotification;
  TOnLog              = TOnIISNotification;
  TOnEndOfNetSession  = TOnIISNotification;
  TOnExtensionTrigger = TOnIISNotification;

  TPortNotification   = (NonSecurePort, SecurePort);
  TPortNotificationSet= Set Of TPortNotification;
  TFilterPriority     = (Default, Low, Medium, High);

  TISAPIFilter = class(Tcomponent)
  private
    { Private declarations }
    F_FilterDescription  : String;
    F_OnGetFilterVersion : TOnGetFilterVersion;
    F_OnReadRawData      : TOnReadRawData;
    F_OnPreprocHeaders   : TOnPreprocHeaders;
    F_OnURLMap           : TOnURLMap;
    F_OnAuthentication   : TOnAuthentication;
    F_OnAuthComplete     : TOnAuthComplete;
    F_OnAccessDenied     : TOnAccessDenied;
    F_OnSendResponce     : TOnSendResponce;
    F_OnSendRawData      : TOnSendRawData;
    F_OnEndOfRequest     : TOnEndOfRequest;
    F_OnLog              : TOnLog;
    F_OnEndOfNetSession  : TOnEndOfNetSession;
    F_OnExtensionTrigger : TOnExtensionTrigger;
    F_OnTerminateFilter  : TOnTerminateFilter;
    F_PortNotification   : TPortNotificationSet;
    F_FilterPriority     : TFilterPriority;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor            Create(AOwner:TComponent); override;
    Destructor             Destroy; override;
    Procedure              ClearLog(Delete:Boolean);
    Procedure              LogInfo(LogData:String);
  published
    { Published declarations }
    Property              FilterDescription   : String               Read F_FilterDescription  Write F_FilterDescription;
    Property              FilterPriority      : TFilterPriority      Read F_FilterPriority     Write F_FilterPriority;
    Property              PortNotification    : TPortNotificationSet Read F_PortNotification   Write F_PortNotification;
    Property              OnGetFilterVersion  : TOnGetFilterVersion  Read F_OnGetFilterVersion Write F_OnGetFilterVersion;
    Property              OnReadRawData       : TOnReadRawData       Read F_OnReadRawData      Write F_OnReadRawData;
    Property              OnPreprocHeaders    : TOnPreprocHeaders    Read F_OnPreprocHeaders   Write F_OnPreprocHeaders;
    Property              OnURLMap            : TOnURLMap            Read F_OnURLMap           Write F_OnURLMap;
    Property              OnAuthentication    : TOnAuthentication    Read F_OnAuthentication   Write F_OnAuthentication;
    Property              OnAuthComplete      : TOnAuthComplete      Read F_OnAuthComplete     Write F_OnAuthComplete;
    Property              OnAccessDenied      : TOnAccessDenied      Read F_OnAccessDenied     Write F_OnAccessDenied;
    Property              OnSendResponce      : TOnSendResponce      Read F_OnSendResponce     Write F_OnSendResponce;
    Property              OnSendRawData       : TOnSendRawData       Read F_OnSendRawData      Write F_OnSendRawData;
    Property              OnEndOfRequest      : TOnEndOfRequest      Read F_OnEndOfRequest     Write F_OnEndOfRequest;
    Property              OnLog               : TOnLog               Read F_OnLog              Write F_OnLog;
    Property              OnEndOfNetSession   : TOnEndOfNetSession   Read F_OnEndOfNetSession  Write F_OnEndOfNetSession;
    Property              OnExtensionTrigger  : TOnExtensionTrigger  Read F_OnExtensionTrigger Write F_OnExtensionTrigger;
    Property              OnTerminateFilter   : TOnTerminateFilter   Read F_OnTerminateFilter  Write F_OnTerminateFilter;
  end;

Var
  TheISAPIFilter : TISAPIFilter;
  Section        : TCriticalSection;
  MainDir        : String;
  MainDll        : String;

Function          GetFilterVersion (var pVer: THTTP_FILTER_VERSION): BOOL stdcall; export;
Function          HttpFilterProc   (var pFc: THTTP_FILTER_CONTEXT; Notificationtype: DWORD; pvNotification: Pointer): DWORD stdcall; export;
Function          TerminateFilter  (dwFlags : DWORD): BOOL stdcall; export;


Procedure         Register;

implementation
Var
 Dll : Array[0..1000] of Char;

{ TISAPIFilter }
constructor TISAPIFilter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if TheISAPIFilter <> Nil Then
     Raise Exception.Create('Only one instance of ISAPIFilter is allowed!');
  TheISAPIFilter       := Self;
  //////////////////////////////////////////////////////////////////////////////
  F_FilterDescription  := '';
  F_PortNotification   := [SecurePort, NonSecurePort];
  F_FilterPriority     := Default;
  //////////////////////////////////////////////////////////////////////////////
  F_OnGetFilterVersion := Nil;
  //////////////////////////////////////////////////////////////////////////////
  F_OnReadRawData      := Nil;
  F_OnPreprocHeaders   := Nil;
  F_OnURLMap           := Nil;
  F_OnAuthentication   := Nil;
  F_OnAuthComplete     := Nil;
  F_OnAccessDenied     := Nil;
  F_OnSendResponce     := Nil;
  F_OnSendRawData      := Nil;
  F_OnEndOfRequest     := Nil;
  F_OnLog              := Nil;
  F_OnEndOfNetSession  := Nil;
  F_OnTerminateFilter  := Nil;
  F_OnExtensionTrigger := Nil;
end;

destructor TISAPIFilter.Destroy;
begin
  TheISAPIFilter := Nil;
  inherited Destroy;
end;

////////////////////////////////////////////////////////////////////////////////
//   Please Use Section.Acquire .. Section.Release when calling this function
////////////////////////////////////////////////////////////////////////////////
Procedure TISAPIFilter.ClearLog(Delete:Boolean);
Var
 OFN : String;
 NFN : String;
 DT  : TDateTime;
Begin
 Try
  OFN := ChangeFileExt(MainDll,'.log');
  NFN := ChangeFileExt(MainDll,'');
  DT  := Yesterday;
  NFN := NFN+'_'+FormatDateTime('yyyymmdd',DT)+'.log';
  RenameFile(OFN,NFN);
  if Delete Then DeleteFile(NFN);
 Except
 End;
End;

////////////////////////////////////////////////////////////////////////////////
//   Please Use Section.Acquire .. Section.Release when calling this function
////////////////////////////////////////////////////////////////////////////////
Procedure TISAPIFilter.LogInfo(LogData:String);
Var
 S  : String;
 FN : String;
 F  : Text;
 DT : TDateTime;
Begin
  {$I-}
  Try
   DT:=Now;
   DateTimeToString(S,'ddddd, tt:zzz',DT);
   S:=S+#9+LogData;
   FN:=ChangeFileExt(MainDll,'.log');
   if FileExists(FN) Then
     Begin
       AssignFile(F,FN);
       Append(F);
     End
   Else
     Begin
        AssignFile(F,FN);
        Rewrite(F);
     End;
   if IOResult = 0 Then
      Begin
        System.Writeln(F,S);
        IOResult;
        CloseFile(F);
        IOResult;
      End;
  Except
  End;
  {$I+}
End;

////////////////////////////////////////////////////////////////////////////////
//                      EXPORTED FUNCIONS REQUIRED BY IIS
////////////////////////////////////////////////////////////////////////////////
function GetFilterVersion(var pVer: THTTP_FILTER_VERSION): BOOL;
begin
 Result := True;
 if Assigned(TheISAPIFilter) Then
    Begin
     Try
       StrPCopy(PVer.lpszFilterDesc,System.Copy(TheISAPIFilter.FilterDescription,1,SF_MAX_FILTER_DESC_LEN));
       /////////////////////////////////////////////////////////////////////////
       PVer.dwFilterVersion                                                    := HTTP_FILTER_REVISION;
       PVer.dwFlags                                                            := SF_NOTIFY_ORDER_DEFAULT;
       if TheISAPIFilter.F_FilterPriority = Low              Then PVer.dwFlags := SF_NOTIFY_ORDER_LOW
       Else
       if TheISAPIFilter.F_FilterPriority = Medium           Then PVer.dwFlags := SF_NOTIFY_ORDER_MEDIUM
       Else
       if TheISAPIFilter.F_FilterPriority = High             Then PVer.dwFlags := SF_NOTIFY_ORDER_HIGH;
       /////////////////////////////////////////////////////////////////////////
       if NonSecurePort In TheISAPIFilter.F_PortNotification Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_NONSECURE_PORT;
       if SecurePort    In TheISAPIFilter.F_PortNotification Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_SECURE_PORT;
       /////////////////////////////////////////////////////////////////////////
       If Assigned(TheISAPIFilter.F_OnReadRawData)           Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_READ_RAW_DATA;
       If Assigned(TheISAPIFilter.F_OnPreprocHeaders)        Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_PREPROC_HEADERS;
       If Assigned(TheISAPIFilter.F_OnURLMap)                Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_URL_MAP;
       If Assigned(TheISAPIFilter.F_OnAuthentication)        Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_AUTHENTICATION;
       If Assigned(TheISAPIFilter.F_OnAuthComplete)          Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_AUTH_COMPLETE;
       If Assigned(TheISAPIFilter.F_OnAccessDenied)          Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_ACCESS_DENIED;
       If Assigned(TheISAPIFilter.F_OnSendResponce)          Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_SEND_RESPONSE;
       If Assigned(TheISAPIFilter.F_OnSendRawData)           Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_SEND_RAW_DATA;
       If Assigned(TheISAPIFilter.F_OnEndOfRequest)          Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_END_OF_REQUEST;
       If Assigned(TheISAPIFilter.F_OnLog)                   Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_LOG;
       If Assigned(TheISAPIFilter.F_OnEndOfNetSession)       Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_END_OF_NET_SESSION;
       If Assigned(TheISAPIFilter.F_OnExtensionTrigger)      Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_EXTENSION_TRIGGER;
       /////////////////////////////////////////////////////////////////////////
       if Assigned(TheISAPIFilter.F_OnGetFilterVersion)      Then TheISAPIFilter.F_OnGetFilterVersion(pVer,Result);
       /////////////////////////////////////////////////////////////////////////
     Except
       Result := False;
     End;
    End;
 {$IFDEF DEBUG}
 Section.Acquire;
 if Assigned(TheISAPIFilter) Then
     TheISAPIFilter.LogInfo('Filter is loaded and GetFilterVersion is called ...');
 Section.Release;
 {$ENDIF}
end;

Function HttpFilterProc (var pFc: THTTP_FILTER_CONTEXT; NotificationType: DWORD; pvNotification: Pointer): DWORD;
Begin
  Result := SF_STATUS_REQ_NEXT_NOTIFICATION;
  if Assigned(TheISAPIFilter) Then
     Begin
      Try
       Case NotificationType of
            SF_NOTIFY_READ_RAW_DATA       : If Assigned(TheISAPIFilter.F_OnReadRawData)      Then TheISAPIFilter.F_OnReadRawData     (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_PREPROC_HEADERS     : If Assigned(TheISAPIFilter.F_OnPreprocHeaders)   Then TheISAPIFilter.F_OnPreprocHeaders  (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_URL_MAP             : If Assigned(TheISAPIFilter.F_OnURLMap)           Then TheISAPIFilter.F_OnURLMap          (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_AUTHENTICATION      : If Assigned(TheISAPIFilter.F_OnAuthentication)   Then TheISAPIFilter.F_OnAuthentication  (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_AUTH_COMPLETE       : If Assigned(TheISAPIFilter.F_OnAuthComplete)     Then TheISAPIFilter.F_OnAuthComplete    (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_ACCESS_DENIED       : If Assigned(TheISAPIFilter.F_OnAccessDenied)     Then TheISAPIFilter.F_OnAccessDenied    (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_SEND_RESPONSE       : If Assigned(TheISAPIFilter.F_OnSendResponce)     Then TheISAPIFilter.F_OnSendResponce    (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_SEND_RAW_DATA       : If Assigned(TheISAPIFilter.F_OnSendRawData)      Then TheISAPIFilter.F_OnSendRawData     (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_END_OF_REQUEST      : If Assigned(TheISAPIFilter.F_OnEndOfRequest)     Then TheISAPIFilter.F_OnEndOfRequest    (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_LOG                 : If Assigned(TheISAPIFilter.F_OnLog)              Then TheISAPIFilter.F_OnLog             (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_END_OF_NET_SESSION  : If Assigned(TheISAPIFilter.F_OnEndOfNetSession)  Then TheISAPIFilter.F_OnEndOfNetSession (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_EXTENSION_TRIGGER   : If Assigned(TheISAPIFilter.F_OnExtensionTrigger) Then TheISAPIFilter.F_OnExtensionTrigger(pFc, NotificationType, pvNotification, Result);
       End;
      Except
        Result := SF_STATUS_REQ_ERROR;
      End;
     End;
  {$IFDEF DEBUG}
  Section.Acquire;
  if Assigned(TheISAPIFilter) Then
     TheISAPIFilter.LogInfo('Filter called for notification No:'+IntToStr(NotificationType));
  Section.Release;
  {$ENDIF}
End;

Function TerminateFilter  (dwFlags : DWORD): BOOL;
Begin
  Result := True;
  if Assigned(TheISAPIFilter) Then
     Begin
      Try
       If Assigned(TheISAPIFilter.F_OnTerminateFilter) Then TheISAPIFilter.F_OnTerminateFilter(dwFlags, Result);
      Except
      End;
     End;
  {$IFDEF DEBUG}
  Section.Acquire;
  if Assigned(TheISAPIFilter) Then
      TheISAPIFilter.LogInfo('Filter is unloaded ...');
  Section.Release;
  {$ENDIF}
End;
////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('Internet', [TISAPIFilter]);
end;

exports
  GetFilterVersion,
  HttpFilterProc,
  TerminateFilter;

Initialization
  TheISAPIFilter      := Nil;
  Section             := TCriticalSection.Create;
  GetModuleFileName   (HInstance,Dll,1000);
  MainDll             := StrPas(Dll);
  MainDir             := ExtractFilePath(MainDll);
Finalization
  TheISAPIFilter := Nil;
  Section.Free;
end.


